//
//  MyNSView.h
//  Graphics Library
//
//  Created by Pooria Azimi on 18/10/11.
//  Copyright 2011 Tehran Polytechnic. All rights reserved.
//


#import "MyNSView.h"

@implementation MyNSView


- (id)initWithFrame:(NSRect)pFrameRect {   
    if (! (self = [super initWithFrame:pFrameRect])) {
		NSLog(@"MyView initWithFrame *Error*");
		return self;
    } // end if
	
	nsRectFrameRect = pFrameRect;
	
	bufferWidth = nsRectFrameRect.size.width;
	bufferHeight = nsRectFrameRect.size.height;
	bufferDepth = 3; // no alpha yet!
	
	imageBuffer = (NSUInteger ***) malloc(bufferWidth * sizeof(NSUInteger **));
	
	for (int i=0; i<bufferWidth; i++) {
		imageBuffer[i] = (NSUInteger **) malloc(bufferHeight * sizeof(NSUInteger *));
		for (int j=0; j<bufferHeight; j++)
			imageBuffer[i][j] = (NSUInteger *) malloc(bufferDepth * sizeof(NSUInteger));
	}
	
	
	// TODO XXXXXXXXX: should free afterwards
	/*
	 @implementation Test
	 int x,y;
	 // Assume x and y get initialised to the "width" and "height" of the array
	 array = (int **)malloc(x*sizeof(int *)); //You need to free((void *)array) AFTER
	 for(int pos = 0; pos < x;pos++){
	 array[pos] = (int *)malloc(y*sizeof(int)); //you loop through array and free((void *)array[pos])
	 }
	 @end
	 */
	
    return self;
}

-(IBAction)clearScreen:(id)sender
{
	[self initializeScreenWithWhiteColor];
	[self refreshScreen];
}


-(IBAction)drawLine:(id)sender
{
	int x0 = [[lineForm cellAtRow:0 column:0] intValue];
	int y0 = [[lineForm cellAtRow:1 column:0] intValue];
	
	int x1 = [[lineForm cellAtRow:2 column:0] intValue];
	int y1 = [[lineForm cellAtRow:3 column:0] intValue];
	
	
	int dx =  abs(x1-x0), sx = x0<x1 ? 1 : -1;
	int dy = -abs(y1-y0), sy = y0<y1 ? 1 : -1; 
	int err = dx+dy, e2; /* error value e_xy */
	
	while (true){
		[self putPixelAtX:x0 Y:y0 WithRed:0 Green:0 Blue:0 Alpha:0];
		if (x0==x1 && y0==y1) break;
		e2 = 2*err;
		if (e2 >= dy) { err += dy; x0 += sx; } /* e_xy+e_x > 0 */
		if (e2 <= dx) { err += dx; y0 += sy; } /* e_xy+e_y < 0 */
	}
	
	[self refreshScreen];
}


-(IBAction)drawCircle:(id)sender
{
	int xm = [[circleForm cellAtRow:0 column:0] intValue];
	int ym = [[circleForm cellAtRow:1 column:0] intValue];
	int r = [[circleForm cellAtRow:2 column:0] intValue];
	
	int x = -r, y = 0, err = 2-2*r; /* II. Quadrant */ 
	do {
		[self putPixelAtX:xm-x Y:ym+y WithRed:0 Green:0 Blue:0 Alpha:0]; /*   I. Quadrant */
		[self putPixelAtX:xm-y Y:ym-x WithRed:0 Green:0 Blue:0 Alpha:0]; /*  II. Quadrant */
		[self putPixelAtX:xm+x Y:ym-y WithRed:0 Green:0 Blue:0 Alpha:0]; /* III. Quadrant */
		[self putPixelAtX:xm+y Y:ym+x WithRed:0 Green:0 Blue:0 Alpha:0]; /*  IV. Quadrant */
		r = err;
		if (r >  x) err += ++x*2+1; /* e_xy+e_x > 0 */
		if (r <= y) err += ++y*2+1; /* e_xy+e_y < 0 */
	} while (x < 0);
	
	[self refreshScreen];
}


-(IBAction)drawEllipse:(id)sender
{
	int x0 = [[ellipseForm cellAtRow:0 column:0] intValue];
	int y0 = [[ellipseForm cellAtRow:1 column:0] intValue];
	
	int x1 = [[ellipseForm cellAtRow:2 column:0] intValue];
	int y1 = [[ellipseForm cellAtRow:3 column:0] intValue];
	
	
	int a = abs(x1-x0), b = abs(y1-y0), b1 = b&1; /* values of diameter */
	long dx = 4*(1-a)*b*b, dy = 4*(b1+1)*a*a; /* error increment */
	long err = dx+dy+b1*a*a, e2; /* error of 1.step */
	
	if (x0 > x1) { x0 = x1; x1 += a; } /* if called with swapped points */
	if (y0 > y1) y0 = y1; /* .. exchange them */
	y0 += (b+1)/2; y1 = y0-b1;   /* starting pixel */
	a *= 8*a; b1 = 8*b*b;
	
	do {
		[self putPixelAtX:x1 Y:y0 WithRed:0 Green:0 Blue:0 Alpha:0]; /*   I. Quadrant */
		[self putPixelAtX:x0 Y:y0 WithRed:0 Green:0 Blue:0 Alpha:0]; /*  II. Quadrant */
		[self putPixelAtX:x0 Y:y1 WithRed:0 Green:0 Blue:0 Alpha:0]; /* III. Quadrant */
		[self putPixelAtX:x1 Y:y1 WithRed:0 Green:0 Blue:0 Alpha:0]; /*  IV. Quadrant */

		e2 = 2*err;
		if (e2 >= dx) { x0++; x1--; err += dx += b1; } /* x step */
		if (e2 <= dy) { y0++; y1--; err += dy += a; }  /* y step */ 
	} while (x0 <= x1);
	
	while (y0-y1 < b) {  /* too early stop of flat ellipses a=1 */
		[self putPixelAtX:x0-1 Y:y0 WithRed:0 Green:0 Blue:0 Alpha:0]; /* -> finish tip of ellipse */
		[self putPixelAtX:x1+1 Y:y0++ WithRed:0 Green:0 Blue:0 Alpha:0];
		[self putPixelAtX:x0-1 Y:y1 WithRed:0 Green:0 Blue:0 Alpha:0];
		[self putPixelAtX:x0+1 Y:y1-- WithRed:0 Green:0 Blue:0 Alpha:0];
	}
	
	[self refreshScreen];
}

- (void)awakeFromNib {
	
	nsBitmapImageRepObj = [[NSBitmapImageRep alloc]
						   initWithBitmapDataPlanes:NULL
										 pixelsWide:nsRectFrameRect.size.width 
										 pixelsHigh:nsRectFrameRect.size.height
									  bitsPerSample:8 
								    samplesPerPixel:3 
										   hasAlpha:NO 
										   isPlanar:NO 
									 colorSpaceName:@"NSCalibratedRGBColorSpace" 
									    bytesPerRow:0 
									   bitsPerPixel:0];
	
	[self initializeScreenWithWhiteColor];
	[self paintBitmap];
}

-(void)initializeScreenWithWhiteColor
{
	for (int i=0; i<bufferWidth; i++)
		for (int j=0; j<bufferHeight; j++)
			for (int d=0; d<bufferDepth; d++)
				imageBuffer[i][j][d] = 255;
}

-(void)putPixelAtX:(int)x Y:(int)y WithRed:(int)red Green:(int)green Blue:(int)blue Alpha:(int)alpha
{
	imageBuffer[x][y][0] = red;
	imageBuffer[x][y][1] = green;
	imageBuffer[x][y][2] = blue;
}

-(void)refreshScreen
{
	[self paintBitmap];
	[self setNeedsDisplay:YES];	
}

- (void) paintBitmap {	
	for (int i=0; i<bufferWidth; i++)
		for (int j=0; j<bufferHeight; j++)
			[nsBitmapImageRepObj setPixel:imageBuffer[i][j] atX:i y:j];
}

- (void)drawRect:(NSRect)pRect {
    
	[NSGraphicsContext saveGraphicsState];
	
	[nsBitmapImageRepObj drawInRect:pRect]; 
	
	[NSGraphicsContext restoreGraphicsState];
	
}

@end